package java.nio.channels.spi;

import java.util.Iterator;
import java.util.ServiceLoader;
import java.util.ServiceConfigurationError;
import java.security.AccessController;
import sun.nio.ch.DefaultAsynchronousChannelProvider;
import java.security.PrivilegedAction;
import java.nio.channels.AsynchronousSocketChannel;
import java.nio.channels.AsynchronousServerSocketChannel;
import java.util.concurrent.ExecutorService;
import java.io.IOException;
import java.nio.channels.AsynchronousChannelGroup;
import java.util.concurrent.ThreadFactory;

public abstract class AsynchronousChannelProvider
{
    private static Void checkPermission() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("asynchronousChannelProvider"));
        }
        return null;
    }
    
    private AsynchronousChannelProvider(Void void1) {
    }
    
    protected AsynchronousChannelProvider() {
        this(checkPermission());
    }
    
    public static AsynchronousChannelProvider provider() {
        return AsynchronousChannelProvider.ProviderHolder.provider;
    }
    
    public abstract AsynchronousChannelGroup openAsynchronousChannelGroup(int p0, ThreadFactory p1) throws IOException;
    
    public abstract AsynchronousChannelGroup openAsynchronousChannelGroup(ExecutorService p0, int p1) throws IOException;
    
    public abstract AsynchronousServerSocketChannel openAsynchronousServerSocketChannel(AsynchronousChannelGroup p0) throws IOException;
    
    public abstract AsynchronousSocketChannel openAsynchronousSocketChannel(AsynchronousChannelGroup p0) throws IOException;
    
    private static class ProviderHolder
    {
        static final AsynchronousChannelProvider provider;
        
        private static AsynchronousChannelProvider load() {
            return (AsynchronousChannelProvider)AccessController.doPrivileged(new PrivilegedAction<AsynchronousChannelProvider>() {
                @Override
                public AsynchronousChannelProvider run() {
                    AsynchronousChannelProvider access$000 = AsynchronousChannelProvider.ProviderHolder.access$000();
                    if (access$000 != null) {
                        return access$000;
                    }
                    AsynchronousChannelProvider access$2 = AsynchronousChannelProvider.ProviderHolder.access$100();
                    if (access$2 != null) {
                        return access$2;
                    }
                    return DefaultAsynchronousChannelProvider.create();
                }
            });
        }
        
        private static AsynchronousChannelProvider loadProviderFromProperty() {
            String property = System.getProperty("java.nio.channels.spi.AsynchronousChannelProvider");
            if (property == null) {
                return null;
            }
            try {
                return (AsynchronousChannelProvider)Class.forName(property, true, ClassLoader.getSystemClassLoader()).newInstance();
            }
            catch (ClassNotFoundException cause) {
                throw new ServiceConfigurationError(null, cause);
            }
            catch (IllegalAccessException cause2) {
                throw new ServiceConfigurationError(null, cause2);
            }
            catch (InstantiationException cause3) {
                throw new ServiceConfigurationError(null, cause3);
            }
            catch (SecurityException cause4) {
                throw new ServiceConfigurationError(null, cause4);
            }
        }
        
        private static AsynchronousChannelProvider loadProviderAsService() {
            Iterator<AsynchronousChannelProvider> iterator = ServiceLoader.load(AsynchronousChannelProvider.class, ClassLoader.getSystemClassLoader()).iterator();
            try {
                return iterator.hasNext() ? ((AsynchronousChannelProvider)iterator.next()) : null;
            }
            catch (ServiceConfigurationError serviceConfigurationError) {
                if ((serviceConfigurationError.getCause() instanceof SecurityException)) {
                    return iterator.hasNext() ? ((AsynchronousChannelProvider)iterator.next()) : null;
                }
                throw serviceConfigurationError;
            }
        }
        
        static AsynchronousChannelProvider access$000() {
            return loadProviderFromProperty();
        }
        
        static AsynchronousChannelProvider access$100() {
            return loadProviderAsService();
        }
        
        static {
            provider = load();
        }
    }
}
